/**
 * {进度条}
 * @author chensbox<chensbox@foxmail.com>
 * @date 2023/04/28 16:14:10
 */

import { css, html, Component, styleMap } from 'wkit'

const RADIUS = 50

class Progress extends Component {
  static props = {
    value: {
      type: Number,
      default: 0,
      observer(val) {
        this.value = this.#fix(val)
      }
    },
    type: 'line' //  line/circle/dashboard
  }

  static styles = [
    css`
      :host {
        display: flex;
      }

      .container {
        position: relative;
        width: 100%;
        -webkit-user-select: none;
        user-select: none;

        &[step='1'] {
          --wc-progress-active-color: var(--wc-progress-active-color-1);
        }
        &[step='2'] {
          --wc-progress-active-color: var(--wc-progress-active-color-2);
        }
        &[step='3'] {
          --wc-progress-active-color: var(--wc-progress-active-color-3);
        }
        &[step='4'] {
          --wc-progress-active-color: var(--wc-progress-active-color-4);
        }
        &[step='5'] {
          --wc-progress-active-color: var(--wc-progress-active-color-5);
        }
      }
    `,
    // line
    css`
      :host([type='line']) {
        width: 100%;

        .progress-value {
          display: none;
          position: absolute;
          z-index: 1;
          top: -32px;
          margin-left: -24px;
          padding: 4px 8px;
          border-radius: 4px;
          font-size: 14px;
          text-align: center;
          background: var(--color-dark-2);
          color: #fff;

          &::after {
            position: absolute;
            left: 50%;
            margin-left: -3px;
            display: block;
            width: 6px;
            height: 6px;
            background: var(--color-dark-2);
            content: '';
            transform: rotate(45deg);
          }
        }

        &:host(:hover) {
          .progress-value {
            display: block;
          }
        }
      }

      .progress-bar {
        display: flex;
        width: 100%;
        height: var(--wc-progress-line-width, 6px);
        border-radius: 32px;
        background: var(--wc-progress-inactive-color, var(--color-plain-2));

        .thumb {
          width: 0;
          height: var(--wc-progress-line-width, 6px);
          border-radius: 32px;
          background: var(--wc-progress-active-color, var(--color-teal-1));
          transition: 0.6s ease;
          transition-property: width, background;
        }
      }
    `,
    // circle & dashboard
    css`
      :host([type='circle']),
      :host([type='dashboard']) {
        width: var(--wc-progress-size, 128px);
        height: var(--wc-progress-size, 128px);
      }
    `
  ]
  // 周长
  #perimeter = 2 * Math.PI * RADIUS

  get trackPath() {
    let isDashboard = this.type === 'dashboard'
    return `
        M 64 64
        m 0 ${isDashboard ? '' : '-'}${RADIUS}
        a ${RADIUS} ${RADIUS} 0 1 1 0 ${isDashboard ? '-' : ''}${RADIUS * 2}
        a ${RADIUS} ${RADIUS} 0 1 1 0 ${isDashboard ? '' : '-'}${RADIUS * 2}
        `
  }
  get trailPathStyle() {
    return styleMap({
      strokeDasharray: `${this.#perimeter * this.rate}px, ${this.#perimeter}px`,
      strokeDashoffset: this.strokeDashoffset
    })
  }

  get strokeDashoffset() {
    let offset = (-1 * this.#perimeter * (1 - this.rate)) / 2
    return `${offset}px`
  }

  get rate() {
    return this.type === 'dashboard' ? 0.75 : 1
  }

  get circlePathStyle() {
    return styleMap({
      strokeDasharray: `${
        this.#perimeter * this.rate * (this.value / 100)
      }px, ${this.#perimeter}px`,
      strokeDashoffset: this.strokeDashoffset,
      transition: 'stroke-dasharray 0.6s ease 0s, stroke 0.6s ease'
    })
  }

  #fix(val) {
    return Math.max(0, Math.min(val, 100))
  }

  #drawCircle() {
    return html`
      <svg viewBox="0 0 128 128">
        <path
          d=${this.trackPath}
          stroke="var(--wc-progress-inactive-color, var(--color-plain-2))"
          stroke-width="var(--wc-progress-line-width, 6px)"
          stroke-linecap="round"
          fill="none"
          style=${this.trailPathStyle}
        />
        <path
          d=${this.trackPath}
          stroke="var(--wc-progress-active-color, var(--color-teal-1))"
          stroke-linecap="round"
          stroke-width="var(--wc-progress-line-width, 6px)"
          fill="none"
          style=${this.circlePathStyle}
        />
      </svg>
    `
  }

  #drawLine() {
    return html`
      <div class="progress-bar">
        <mark
          class="thumb"
          style=${styleMap({ width: this.value + '%' })}
        ></mark>
      </div>
      <div class="progress-value" style=${styleMap({ left: this.value + '%' })}>
        ${this.value + '%'}
      </div>
    `
  }

  render() {
    let step = ~~(this.value / 20) || 1
    return html`
      <main class="container" step=${step}>
        ${this.type === 'line' ? this.#drawLine() : this.#drawCircle()}
      </main>
    `
  }
}

Progress.reg('progress')
